/*
	* SA_ReVision shader file
	* Modifications by ZIMMER
	* Original DOF code by Marty McFly

	Please ask permissions from SA_ReVision 
	Team if you want re-use this shader!
*/

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//UI vars, nothing to edit for standard users below this point
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

bool    bADOF_AutofocusEnable 		< string UIName="DOF: Enable Autofocus";															> = {true};
float2	fADOF_AutofocusCenter 		< string UIName="DOF: Autofocus sample center"; 	string UIWidget="Spinner"; 	float UIStep=0.01; 	float UIMin=0.00; 	float UIMax=1.00;	> = {0.5,0.5};
float	fADOF_AutofocusRadius		< string UIName="DOF: Autofocus sample radius";		string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=0.01;	float UIMax=1.00;	> = {0.05};
float	fADOF_ManualfocusDepth		< string UIName="DOF: Manual focus depth";		string UIWidget="Spinner";	float UIStep=0.001;	float UIMin=0.00;	float UIMax=1.0;	> = {0.05};
float	fADOF_NearBlurCurve		< string UIName="DOF: Near blur curve";			string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=0.01;	float UIMax=20.0;	> = {1.0};
float	fADOF_FarBlurCurve		< string UIName="DOF: Far blur curve";			string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=0.01;	float UIMax=20.0;	> = {1.4};
float	fADOF_HyperFocus		< string UIName="DOF: Hyperfocal depth distance";	string UIWidget="Spinner";	float UIStep=0.001;	float UIMin=0.00;	float UIMax=1.0;	> = {0.015};
float	fADOF_RenderResolutionMult	< string UIName="DOF: Blur render res mult";		string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=0.50;	float UIMax=1.0;	> = {0.5};
float	fADOF_BokehIntensity		< string UIName="DOF: Bokeh Intensity";			string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=0.0;	float UIMax=1.0;	> = {0.5};
float	fADOF_ShapeRadius		< string UIName="DOF: Bokeh shape max size";		string UIWidget="Spinner";	float UIStep=0.1;	float UIMin=0.0;	float UIMax=100.0;	> = {15.0};
int	iADOF_ShapeVertices		< string UIName="DOF: Bokeh shape vertices";		string UIWidget="spinner";				int UIMin=3;		int UIMax=9;		> = {6};
int	iADOF_ShapeQuality		< string UIName="DOF: Bokeh shape quality";		string UIWidget="spinner";				int UIMin=2;		int UIMax=25;		> = {5};
float	fADOF_ShapeCurvatureAmount	< string UIName="DOF: Bokeh shape roundness";		string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=-1.0;	float UIMax=1.0;	> = {1.0};
float	fADOF_ShapeRotation		< string UIName="DOF: Bokeh shape rotation (\xB0)";	string UIWidget="Spinner";	float UIStep=1;		float UIMin=0;		float UIMax=360;	> = {15};
float	fADOF_ShapeAnamorphRatio	< string UIName="DOF: Bokeh shape aspect ratio";	string UIWidget="Spinner";	float UIStep=0.01;	float UIMin=0.0;	float UIMax=1.0;	> = {1.0};
float	EMotionBlurStepsInv		< string UIName="Motion_Blur_Level";			string UIWidget="Spinner";				float UIMin=0.0;	float UIMax=5.0;	> = {1.00};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//external enb parameters, do not modify
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4	tempF1; 		//0,1,2,3
float4  tempF2;                 //5,6,7,8
float4	tempF3; 		//9,0
float4	Timer; 			//x = generic timer in range 0..1, period of 16777216 ms (4.6 hours), y = average fps, w = frame time elapsed (in seconds)
float4	ScreenSize; 		//x = Width, y = 1/Width, z = aspect, w = 1/aspect, aspect is Width/Height
float	ENightDayFactor;	//changes in range 0..1, 0 means that night time, 1 - day time
float	EInteriorFactor;	//changes 0 or 1. 0 means that exterior, 1 - interior
float	FieldOfView;            //fov in degrees
float	FadeFactor;             //adaptation delta time for focusing

//transposed transform matrix view*projection and inverse of it
float4 MatrixVP[4];
float4 MatrixInverseVP[4];
float4 MatrixVPRotation[4];
float4 MatrixInverseVPRotation[4];
float4 MatrixView[4];
float4 MatrixInverseView[4];
float4 CameraPosition;

float4 PrevMatrixVPRotation[4];
float4 PrevMatrixInverseVPRotation[4];
float4 PrevCameraPosition;

float	EnableDepthOfField;
float 	EnableMotionBlur;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Semi-hardcoded parameters, DO NOT MODIFY unless you know what you do.
//But what am I saying, you're gonna do it anyways.
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define DISCRADIUS_RESOLUTION_BOUNDARY_LOWER 	0.25//1.0	//used for blending blurred scene.
#define DISCRADIUS_RESOLUTION_BOUNDARY_UPPER 	1.0//6.0	//used for blending blurred scene.
#define DISCRADIUS_RESOLUTION_BOUNDARY_CURVE    0.5		//used for blending blurred scene.
#define FPS_HAND_BLUR_CUTOFF_DIST		0.3353		//fps hand depth (x10.000), change if you perceive blurred fps weapons.
#define FPS_HAND_BLUR_CUTOFF_CHECK		0		//blur = max if depth > hand depth, else 0, useful for tweaking above param
#define GAUSSIAN_BUILDUP_MULT			4.0		//value of x -> gaussian reaches max radius at |CoC| == 1/x

static const float2 PixelSize 			= float2(ScreenSize.y,ScreenSize.y*ScreenSize.z);
static const float2 ViewPortSize 		= float2(ScreenSize.x,ScreenSize.x*ScreenSize.w);

#define linearstep(a,b,x) saturate((x-a)/(b-a))
#define LinearizeDepth(x) x *= rcp(-2999.0 * x + 3000.0)

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//mod parameters, do not modify
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

texture2D texOriginal;  //backbuffer, before any changes made by this file
texture2D texMask;      //alpha channel is mask for skinned objects (less than 1) and amount of sss
texture2D texColor;     //backbuffer, output of previous pass
texture2D texDepth;     //scene depth
texture2D texNormal;
texture2D texNoise;     //256x256 random white noise
texture2D texPalette;   //enbpalette.bmp
texture2D texFocus;     //computed focusing depth
texture2D texCurr;      //4*4 texture for focusing
texture2D texPrev;      //4*4 texture for focusing

sampler2D SamplerOriginal = sampler_state
{
	Texture   = <texOriginal>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerMask = sampler_state
{
	Texture   = <texMask>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerColor = sampler_state
{
	Texture   = <texColor>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerDepth = sampler_state
{
	Texture   = <texDepth>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerNormal = sampler_state
{
	Texture   = <texNormal>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerNoise = sampler_state
{
	Texture   = <texNoise>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU  = Wrap;
	AddressV  = Wrap;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerPalette = sampler_state
{
	Texture   = <texPalette>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerCurr = sampler_state
{
	Texture   = <texCurr>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerPrev = sampler_state
{
	Texture   = <texPrev>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerFocus = sampler_state
{
	Texture   = <texFocus>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Vertex Shader
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

struct VS_OUTPUT_DOF
{
	float4 vpos  : POSITION;
	float4 txcoord : TEXCOORD0;
        float4 offset0 : TEXCOORD1;
        float2x2 offsmat : TEXCOORD2;
};

struct VS_OUTPUT_POST
{
	float4 vpos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

struct VS_INPUT_POST
{
	float3 pos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

VS_OUTPUT_DOF VS_DoF(VS_INPUT_POST IN, uniform float scale)
{
        VS_OUTPUT_DOF OUT;
        OUT.vpos    = float4(IN.pos,1.0);
        OUT.txcoord.xy = IN.txcoord.xy;
        OUT.txcoord.zw = IN.txcoord.xy / scale;

        //can't precompute vertices directly, not enough registers in sm3
        sincos(radians(fADOF_ShapeRotation), OUT.offset0.x, OUT.offset0.y);
        sincos(6.2831853 / iADOF_ShapeVertices, OUT.offsmat._21, OUT.offsmat._22);
        OUT.offsmat._11 = OUT.offsmat._22;
        OUT.offsmat._12 = -OUT.offsmat._21;

        OUT.offset0.zw = mul(OUT.offset0.xy, OUT.offsmat);

        return OUT;
}

VS_OUTPUT_POST VS_Quad(VS_INPUT_POST IN)
{
	VS_OUTPUT_POST OUT;
	float4 pos = float4(IN.pos.x,IN.pos.y,IN.pos.z,1.0);
	if (EnableMotionBlur==0.0) pos.xy = 10000.0;
	OUT.vpos = pos;
	OUT.txcoord.xy = IN.txcoord.xy;
	return OUT;
}


float FuncMasking(float4 uv)
{
	float mask = tex2Dlod(SamplerNormal, uv).w;
	if ((mask>=(50.0/255.0)) && (mask!=1.0))
		mask=0.0;
	else mask = 1.0;
	return mask;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Functions
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float GetLinearDepth(float2 texcoord)
{
	float depth = tex2Dlod(SamplerDepth, texcoord.xyxy).x;
	LinearizeDepth(depth);
	return depth;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float CircleOfConfusion(float2 texcoord, bool aggressiveLeakReduction)
{
	float2 depthdata; //x - linear scene depth, y - linear scene focus
	float scenecoc;   //blur value, signed by position relative to focus plane

        depthdata.x = GetLinearDepth(texcoord.xy);

	[branch]
	if(aggressiveLeakReduction)
	{
                float3 neighbourOffsets = float3(PixelSize.xy, 0);
                float4 neighbourDepths = float4(tex2D(SamplerDepth, texcoord.xy - neighbourOffsets.xz).x, //left
                                                tex2D(SamplerDepth, texcoord.xy + neighbourOffsets.xz).x, //right
                                                tex2D(SamplerDepth, texcoord.xy - neighbourOffsets.zy).x, //top
                                                tex2D(SamplerDepth, texcoord.xy + neighbourOffsets.zy).x);//bottom

		LinearizeDepth(neighbourDepths);

		float neighbourMin = min(min(neighbourDepths.x,neighbourDepths.y),min(neighbourDepths.z,neighbourDepths.w));
		depthdata.x = lerp(min(neighbourMin, depthdata.x), depthdata.x, 0.001);
	}

	depthdata.y = tex2D(SamplerFocus, texcoord.xy).x;
	float handdepth = depthdata.x;

	depthdata.xy = saturate(depthdata.xy / fADOF_HyperFocus);

	[branch]
	if(depthdata.x < depthdata.y)
	{
		scenecoc = depthdata.x / depthdata.y - 1.0;
		scenecoc = ldexp(scenecoc, -0.5*fADOF_NearBlurCurve*fADOF_NearBlurCurve);
	}
	else
	{
		scenecoc = (depthdata.x - depthdata.y)/(ldexp(depthdata.y, fADOF_FarBlurCurve*fADOF_FarBlurCurve) - depthdata.y);
	        scenecoc = saturate(scenecoc);
	}

	return scenecoc;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Pixel Shaders
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//fullres -> 4x4
float4	PS_ReadFocus(VS_OUTPUT_POST IN) : COLOR
{
	float scenefocus = 0.0;

        [branch]
	if(bADOF_AutofocusEnable == true)
	{
		float samples = 10.0;
		float weightsum = 1e-6;

		for(float xcoord = 0.0; xcoord < samples; xcoord++)
		for(float ycoord = 0.0; ycoord < samples; ycoord++)
		{
			float2 sampleOffset = (float2(xcoord,ycoord) + 0.5) / samples;
			sampleOffset = sampleOffset * 2.0 - 1.0;
			sampleOffset *= fADOF_AutofocusRadius;
			sampleOffset += (fADOF_AutofocusCenter - 0.5);

			float sampleWeight = saturate(1.2 * exp2(-dot(sampleOffset,sampleOffset)*4.0));

			float tempfocus = GetLinearDepth(sampleOffset * 0.5 + 0.5);
			sampleWeight *= rcp(tempfocus + 0.001);

			sampleWeight *= saturate(tempfocus > FPS_HAND_BLUR_CUTOFF_DIST * 1e-4); //remove fps hands from focus calculations

			scenefocus += tempfocus * sampleWeight;
			weightsum += sampleWeight;
		}
		scenefocus /= weightsum;
	}
	else
	{
		scenefocus = fADOF_ManualfocusDepth;
	}

	return scenefocus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//4x4 -> 1x1
float4	PS_WriteFocus(VS_OUTPUT_POST IN) : COLOR
{
        float prevFocus         = 0;
	float currFocus         = 0;
        float4 sampleCoord      = 0;

	for(int i = 0; i < 4; i++)
        {
	        for(int j = 0; j < 4; j++)
	        {
                        prevFocus += tex2Dlod(SamplerPrev, sampleCoord).x;
                        currFocus += tex2Dlod(SamplerCurr, sampleCoord).x;
                        sampleCoord.y += 0.25;
                }
                sampleCoord.y = 0;
                sampleCoord.x += 0.25;
	}

	return (bADOF_AutofocusEnable == true) ? lerp(prevFocus,currFocus,FadeFactor) * 0.0625 : currFocus * 0.0625;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4	PS_CoC(VS_OUTPUT_POST IN) : COLOR
{
	float4 color = tex2D(SamplerColor, IN.txcoord.xy);

	static const float2 sampleOffsets[4] = {float2( 1.5, 0.5) * PixelSize.xy,
		                                float2( 0.5,-1.5) * PixelSize.xy,
				                float2(-1.5,-0.5) * PixelSize.xy,
				                float2(-0.5, 1.5) * PixelSize.xy};

	float4 compColor = 0.0;
	float centerDepth = GetLinearDepth(IN.txcoord.xy);
        float4 sampleCoord = 0.0;
        float3 neighbourOffsets = float3(PixelSize.xy, 0);

	[loop]
	for(int i=0; i<4; i++)
	{
		sampleCoord.xy = IN.txcoord.xy + sampleOffsets[i];

		float3 sampleColor = tex2Dlod(SamplerColor,  sampleCoord).rgb;

                float4 sampleDepths = float4(tex2Dlod(SamplerDepth, sampleCoord + neighbourOffsets.xzzz).x,  //right
                                             tex2Dlod(SamplerDepth, sampleCoord - neighbourOffsets.xzzz).x,  //left
                                             tex2Dlod(SamplerDepth, sampleCoord + neighbourOffsets.zyzz).x,  //bottom
                                             tex2Dlod(SamplerDepth, sampleCoord - neighbourOffsets.zyzz).x); //top

                float sampleDepthMin = min(min(sampleDepths.x,sampleDepths.y),min(sampleDepths.z,sampleDepths.w));
                LinearizeDepth(sampleDepthMin);

		sampleColor /= 1.0 + max(max(sampleColor.r, sampleColor.g), sampleColor.b);

		float sampleWeight = saturate(sampleDepthMin * rcp(centerDepth) + 1e-3);
		compColor += float4(sampleColor.rgb * sampleWeight, sampleWeight);
	}

	compColor.rgb /= compColor.a;
	compColor.rgb /= 1.0 - max(compColor.r, max(compColor.g, compColor.b));

	color.rgb = lerp(color.rgb, compColor.rgb, saturate(compColor.w * 8.0));
	color.w = CircleOfConfusion(IN.txcoord.xy, 1);

	return color;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void ShapeRoundness(inout float2 sampleOffset, in float roundness)
{
	sampleOffset *= (1.0-roundness) + rsqrt(dot(sampleOffset,sampleOffset))*roundness;
}

float2 CoC2BlurRadius(float CoC)
{
	return float2(fADOF_ShapeAnamorphRatio,ScreenSize.z) * CoC * fADOF_ShapeRadius * 6e-4;
}

float4	PS_DoF_Main(VS_OUTPUT_DOF IN) : COLOR
{
	clip(1.01-max(IN.txcoord.z,IN.txcoord.w));

	float4 BokehSum, BokehMax;
	BokehMax = BokehSum		= tex2D(SamplerColor, IN.txcoord.zw);
	float weightSum 		= 1.0;
	float CoC 			= abs(BokehSum.w);
	float2 bokehRadiusScaled	= CoC2BlurRadius(CoC);
	float nRings 			= lerp(1.0,iADOF_ShapeQuality,saturate(CoC)) + frac(dot(IN.txcoord.xy, ViewPortSize) * 0.5) * 0.5;

	if(bokehRadiusScaled.x < DISCRADIUS_RESOLUTION_BOUNDARY_LOWER * ScreenSize.y) return BokehSum;

	bokehRadiusScaled /= nRings;
	CoC /= nRings;

	[fastopt]
        for (int iVertices = 0; iVertices < iADOF_ShapeVertices; iVertices++)
        {
	        [fastopt]
                for(float iRings = 1; iRings <= nRings; iRings++)
                {
                        [fastopt]
                        for(float iSamplesPerRing = 0; iSamplesPerRing < iRings; iSamplesPerRing++)
	                {
		              float2 sampleOffset = lerp(IN.offset0.xy,IN.offset0.zw,iSamplesPerRing/iRings);
		              ShapeRoundness(sampleOffset,fADOF_ShapeCurvatureAmount);

		              float4 sampleBokeh 	= tex2Dlod(SamplerColor, float4(IN.txcoord.zw + sampleOffset.xy * (bokehRadiusScaled * iRings),0,0));
		              float sampleWeight	= saturate(1e6 * (abs(sampleBokeh.a) - CoC * (float)iRings) + 1.0);
		              sampleBokeh.rgb 	*= sampleWeight;
		              weightSum 	+= sampleWeight;
		              BokehSum 		+= sampleBokeh;
		              BokehMax 		= max(BokehMax,sampleBokeh);
	               }
               }

               IN.offset0.xy = IN.offset0.zw;
               IN.offset0.zw = mul(IN.offset0.zw, IN.offsmat);
       }

       return lerp(BokehSum / weightSum, BokehMax, fADOF_BokehIntensity);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4	PS_DoF_Combine(VS_OUTPUT_POST IN) : COLOR
{

	float4 blurredColor = tex2D(SamplerColor, IN.txcoord.xy * fADOF_RenderResolutionMult);
	float4 originalColor  = tex2D(SamplerOriginal, IN.txcoord.xy);

	float CoC 		= CircleOfConfusion(IN.txcoord.xy, 0);
	float CoCblurred	= blurredColor.a;

	float bokehRadiusPixels = abs(CoC2BlurRadius(CoC).x * ScreenSize.x);

	float blendWeight = linearstep(DISCRADIUS_RESOLUTION_BOUNDARY_LOWER, DISCRADIUS_RESOLUTION_BOUNDARY_UPPER, bokehRadiusPixels);
	      blendWeight = pow(blendWeight,DISCRADIUS_RESOLUTION_BOUNDARY_CURVE);

	float4 outColor;
	outColor.rgb    = lerp(originalColor.rgb, blurredColor.rgb, blendWeight);
	outColor.a      = saturate(abs(lerp(CoC, CoCblurred, blendWeight)) * GAUSSIAN_BUILDUP_MULT);

	return outColor;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4	PS_DoF_Gauss(VS_OUTPUT_POST y) : SV_Target
{
	float2 coord = y.txcoord.xy;
	float2 pixelSize = ScreenSize.y;
	pixelSize.y *= ScreenSize.z;
	float4 origcolor = tex2D(SamplerColor, coord.xy);
	
	float blurAmount = abs(origcolor.w * 2.0 - 1.0) * 3.0;
	blurAmount /= 10 * (1 + ((3.0 - 0) % 3));

	float weight[5] = {0.227027, 0.194595, 0.121622, 0.0540541, 0.0162162};
	float4 color = origcolor * weight[0];

	for(int i=1; i < 5; i++)
	{
		color += tex2D(SamplerColor, coord + float2(i * pixelSize.x * blurAmount * 0, 0)) * weight[i];
		color += tex2D(SamplerColor, coord - float2(i * pixelSize.x * blurAmount * 0, 0)) * weight[i];
	}
	color.a = 1;
	return color;
}

float4 PS_MotionBlur(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	float4	res;
	float4	coord;
	coord.xy=IN.txcoord.xy;
	coord.zw=0.0;
    float seed = 0.0;
	float4	color=tex2D(SamplerOriginal, IN.txcoord.xy);
	color.xyz*=color.xyz;
	float	scenedepth=tex2D(SamplerDepth, IN.txcoord.xy).x;
	float4	tempvec;
	float4	temppos;
	float4	worldpos;
	float4	wpos;	
	float4	prevworldpos;
	float4	prevscreen;
	float4	currscreen;
	currscreen.xy=IN.txcoord.xy;

	tempvec.xy=IN.txcoord.xy*2.0-1.0;
	tempvec.y=-tempvec.y;
	tempvec.z=scenedepth;
	tempvec.w=1.0;

	worldpos.x=dot(tempvec, MatrixInverseVPRotation[0]);
	worldpos.y=dot(tempvec, MatrixInverseVPRotation[1]);
	worldpos.z=dot(tempvec, MatrixInverseVPRotation[2]);
	worldpos.w=dot(tempvec, MatrixInverseVPRotation[3]);
	worldpos.xyz/=worldpos.w;
	
    wpos.x = dot(tempvec, MatrixInverseVPRotation[0]);
    wpos.y = dot(tempvec, MatrixInverseVPRotation[1]);
    wpos.z = dot(tempvec, MatrixInverseVPRotation[2]);
    wpos.w = dot(tempvec, MatrixInverseVPRotation[3]);
    wpos.xyz/= wpos.w;	
   
    float4 wp = wpos;
    float pow2 = length(wp.xyz);
    float distance = 0.0;
    float3 dp1 = -1.0/exp(length(distance-pow2)/(2.0*0.48)); 		 

	float	distfact=dot(worldpos.xyz, worldpos.xyz);
	distfact=saturate(distfact*0.0002);
	distfact*=distfact;
	
	float	distfact2=dot(worldpos.xyz, worldpos.xyz);
	distfact2=saturate(distfact2*0.0002);
	distfact2*=distfact2;
	distfact2 = pow(distfact2, 117000000.0*24.50);
	distfact2 = lerp(1.0, distfact2, dp1); 
	
    float4 vfog2 = pow(distfact2, -1000000.0*24.50);	
   
	worldpos.xyz+=CameraPosition;
	prevworldpos.x=dot(tempvec, PrevMatrixInverseVPRotation[0]);
	prevworldpos.y=dot(tempvec, PrevMatrixInverseVPRotation[1]);
	prevworldpos.z=dot(tempvec, PrevMatrixInverseVPRotation[2]);
	prevworldpos.w=dot(tempvec, PrevMatrixInverseVPRotation[3]);
	prevworldpos.xyz/=prevworldpos.w;

	prevworldpos.xyz+=PrevCameraPosition;

	float3 dir;
	dir = (worldpos.xyz-prevworldpos.xyz);
	dir = dir * 0.70;
	prevworldpos.xyz = prevworldpos.xyz+dir.xyz;

	tempvec.xyz=prevworldpos.xyz;
	tempvec.w=1.0;
	prevscreen.x=dot(tempvec, MatrixVP[0]);
	prevscreen.y=dot(tempvec, MatrixVP[1]);
	prevscreen.z=dot(tempvec, MatrixVP[2]);
	prevscreen.w=dot(tempvec, MatrixVP[3]);
	prevscreen.xyz/=prevscreen.w;
	prevscreen.xy=prevscreen.xy*float2(0.5,-0.5) + 0.5;
	prevscreen.z=1.0/max(1.0-prevscreen.z, 0.0000000000001);
	currscreen.z=1.0/max(1.0-scenedepth, 0.0000000000001);

	float mask;
	      mask=FuncMasking(coord);		  
	float4 no1 = tex2Dlod(SamplerNormal, coord).w;	
	if (no1.x>=(25.0/1.0)) no1.z=24.50;	
	float2	jitteruv;
	        jitteruv=frac((vPos.xy+seed)*0.0625)+0.03125;
	float4	jitter=tex2D(SamplerNoise, jitteruv);
	float	step=-0.20*EMotionBlurStepsInv;	        
	float	shift=step*jitter.y;
	float	weight=1.0;
	int		i=0;
	while ((shift<1.0) && (i<12))
	{
		i++;
		float	tempmask;		
		float4	tempcolor;
		float	tempweight;
		coord.xyz=lerp(currscreen.xyz, prevscreen.xyz, shift);

		tempmask=FuncMasking(coord);

		if (mask==0.0)
		{
			float	tempdepth=tex2Dlod(SamplerDepth, coord).x;
			tempdepth=1.0/max(1.0-tempdepth, 0.0000000000001);
			tempmask*=saturate((coord.z-tempdepth)*1000.0);
		}

		tempmask=saturate(tempmask+vfog2);
		shift+=step;
		float2	uvfact=saturate(-coord.xy)+saturate(coord.xy-1.0);
		uvfact.x+=uvfact.y;
		tempmask=saturate(tempmask - uvfact.x*100000.0);

		tempcolor=tex2Dlod(SamplerColor, coord);
		tempcolor.xyz*=tempcolor.xyz;

		color+=tempcolor * tempmask;
		weight+=tempmask;
	}
	color/=weight;
	color.xyz=pow(color.xyz, 0.5);
	res.xyz=color;
	res.w=1.0;
	return res;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Techniques
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define TECH9(NAME, VERTEXSHADER, PIXELSHADER)\
technique NAME { pass p0 { VertexShader = compile vs_3_0 VERTEXSHADER; PixelShader  = compile ps_3_0 PIXELSHADER; }}

technique ReadFocus
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_ReadFocus();
		
	}
}

technique WriteFocus
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_WriteFocus();
		
	}
}

technique PostProcess
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_CoC();
		
	}
}

technique PostProcess2
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_DoF(fADOF_RenderResolutionMult);
		PixelShader  = compile ps_3_0 PS_DoF_Main();
		
	}
}

technique PostProcess3
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_DoF_Combine();
		
	}
}

technique PostProcess4
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_DoF_Gauss();
		
	}
}

technique PostProcess5
{
	pass p0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_MotionBlur();
	}
}
